@@ -1,8 +1,4 @@
-/** \file input_common.h
-
-Header file for the low level input library
-
-*/
+// Header file for the low level input library.
 #ifndef INPUT_COMMON_H
 #define INPUT_COMMON_H
 
@@ -10,18 +6,17 @@ Header file for the low level input library
 
 #include "common.h"
 
-enum
-{
+enum {
     R_MIN = INPUT_COMMON_BASE,
-    // R_NULL is sometimes returned by the input when a character was requested
-    // but none could be delivered, or when an exception happened.
+    // R_NULL is sometimes returned by the input when a character was requested but none could be
+    // delivered, or when an exception happened.
     R_NULL = R_MIN,
     R_EOF,
-    // Key codes for inputrc-style keyboard functions that are passed on
-    // to the caller of input_read().
+    // Key codes for inputrc-style keyboard functions that are passed on to the caller of
+    // input_read().
     //
-    // NOTE: If you modify this sequence of symbols you must update the
-    // name_arr, code_arr and desc_arr variables in input.cpp to match!
+    // NOTE: If you modify this sequence of symbols you must update the name_arr, code_arr and
+    // desc_arr variables in input.cpp to match!
     R_BEGINNING_OF_LINE,
     R_END_OF_LINE,
     R_FORWARD_CHAR,
@@ -75,23 +70,20 @@ enum
     R_BACKWARD_JUMP,
     R_AND,
     R_CANCEL,
+    R_TIMEOUT,  // we didn't get interactive input within wait_on_escape_ms
     R_MAX = R_CANCEL,
-    // This is a special psuedo-char that is not used other than to mark the
-    // end of the the special characters so we can sanity check the enum range.
+    // This is a special psuedo-char that is not used other than to mark the end of the the special
+    // characters so we can sanity check the enum range.
     R_SENTINAL
 };
 
-/**
-   Init the library
-*/
+/// Init the library.
 void input_common_init(int (*ih)());
 
-/**
-   Free memory used by the library
-*/
+/// Free memory used by the library.
 void input_common_destroy();
 
-// Adjust the escape timeout.
+/// Adjust the escape timeout.
 void update_wait_on_escape_ms();
 
 /// Set the escape timeout directly.
@@ -101,24 +93,20 @@ void set_wait_on_escape_ms(int ms);
 /// convert them to a wchar_t. Conversion is done using mbrtowc. If a character has previously been
 /// read and then 'unread' using \c input_common_unreadch, that character is returned. If timed is
 /// true, readch2 will wait at most WAIT_ON_ESCAPE milliseconds for a character to be available for
-/// reading before returning with the value WEOF.
+/// reading before returning with the value R_EOF.
 wchar_t input_common_readch(int timed);
 
-/**
-   Enqueue a character or a readline function to the queue of unread
-   characters that input_readch will return before actually reading from fd
-   0.
-*/
+/// Enqueue a character or a readline function to the queue of unread characters that input_readch
+/// will return before actually reading from fd 0.
 void input_common_queue_ch(wint_t ch);
 
-/**
-   Add a character or a readline function to the front of the queue of unread
-   characters.  This will be the first character returned by input_readch
-   (unless this function is called more than once).
-*/
+/// Add a character or a readline function to the front of the queue of unread characters.  This
+/// will be the first character returned by input_readch (unless this function is called more than
+/// once).
 void input_common_next_ch(wint_t ch);
 
-/** Adds a callback to be invoked at the next turn of the "event loop." The callback function will be invoked and passed arg. */
+/// Adds a callback to be invoked at the next turn of the "event loop." The callback function will
+/// be invoked and passed arg.
 void input_common_add_callback(void (*callback)(void *), void *arg);
 
 #endif
